<?php
namespace Service;

use Doctrine\ORM\EntityManager;
use Exception\CustomException;
use Util\Messages;
use Util\Utilities;
use Zeedhi\Framework\HTTP\Kernel;
use Util\InstanceProvider;
use \Zeedhi\DevOps\Model\Projectmanagement\Entities;

class Attachment {


    /** @var  EntityManager $entityManager */
    protected $entityManager;
    /** @var  \Factory\CallAttachment $callAttachmentFactory */
    protected $callAttachmentFactory;
    /** @var Kernel $httpKernel */
    protected $httpKernel;
    /** @var \Service\Integration\PMIntegration $pmIntegrationService */
    protected $pmIntegrationService;
    /** @var  \Service\Integration\CRMIntegration $crmIntegrationService */
    protected $crmIntegrationService;
    /** @var \Teknisa\Model\Teknisa\Repositories\CpCall $cpCallRepository */
    protected $cpCallRepository;
    /** @var \Teknisa\Model\Teknisa\Repositories\CpCallAttachment $cpCallAttachmentRepository */
    protected $cpCallAttachmentRepository;
    /** @var \Teknisa\Model\Geral\Repositories\Operador $operadorRepository */
    protected $operadorRepository;
    /** @var \Teknisa\Model\Crm\Repositories\Contato $contatoRepository */
    protected $contatoRepository;
    /** @var \Teknisa\Model\Geral\Repositories\Paramgeral $paramGeralRepository */
    protected $paramGeralRepository;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Attachment attachmentRepository */
    protected $attachmentRepository;

    protected $DIRECTORY_BASIC_UPLOAD;
    /**
     * @var \Util\Utilities
     */
    private $environment;
    /**
     * @var CallComment
     */
    private $commentService;
    /**
     * @var Integration\INTRANETIntegration
     */
    private $INTRANETIntegration;
    private $attachmentsBaseDirectory;

    public function __construct(EntityManager $entityManager, \Util\Utilities $environment, Kernel $httpKernel,
                                Integration\PMIntegration $pmIntegrationService,
                                Integration\CRMIntegration $crmIntegrationService,
                                \Factory\CallAttachment $callAttachmentFactory, CallComment $commentService,
                                Integration\INTRANETIntegration $INTRANETIntegration,
                                $attachmentsBaseDirectory) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->httpKernel = $httpKernel;
        $this->pmIntegrationService = $pmIntegrationService;
        $this->crmIntegrationService = $crmIntegrationService;
        $this->callAttachmentFactory = $callAttachmentFactory;

        $this->cpCallRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\CpCall');
        $this->cpCallAttachmentRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\CpCallAttachment');
        $this->operadorRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Operador');
        $this->contatoRepository = $this->entityManager->getRepository('\Teknisa\Model\Crm\Entities\Contato');
        $this->paramGeralRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Paramgeral');
        $this->attachmentRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Attachment');

        $this->DIRECTORY_BASIC = $attachmentsBaseDirectory;
        $this->DIRECTORY_BASIC_UPLOAD = $this->DIRECTORY_BASIC.DIRECTORY_SEPARATOR.'attachments';
        $this->commentService = $commentService;
        $this->INTRANETIntegration = $INTRANETIntegration;
    }

    /**
     * @param \Zeedhi\DevOps\Model\Projectmanagement\Entities\Attachment $attachment
     * @param \Teknisa\Model\Teknisa\Entities\CpCallAttachment $callAttachment
     * @return array
     */
    public function getParamsForDownload($attachment, $callAttachment = null)
    {
        if ($attachment) {
            $name = $attachment->getLabel();
            $size = $attachment->getSizeFile();
            $type = $attachment->getTypeFile();
            $fileName = $this->pmIntegrationService->fileServerFilesUrl.$attachment->getFileName();
        } else {
            $name = $callAttachment->getLabel();
            $size = $callAttachment->getFilesize();
            $type = $callAttachment->getFiletype();
            $fileName = $callAttachment->getFileName();
        }
        return array($name, $size, $type, $fileName);
    }

    /**
     * @param $type
     * @param $size
     * @param $name
     * @param $fileName
     * @param bool $isAttachment
     */
    public function setParamsForDownload($type, $size, $name, $fileName,$isAttachment = true)
    {
        header('Accept-Ranges: none');
        header("Content-Type: " . $type);
        if($size){
            header("Content-Length: " . $size);
        }

        if($isAttachment){
            header('Content-Disposition: attachment; filename="' . $name.'"');
        }else{
            header('Content-Disposition: inline;filename="' . $name.'"');
        }
        readfile($fileName);
    }

    /**
     * @param \Teknisa\Model\Teknisa\Entities\CpCallAttachment[] $callAttachments
     * @param $index
     * @param $basic
     * @param $fileData
     * @param \Teknisa\Model\Teknisa\Entities\CpCall $call
     * @return mixed
     */
    public function setFactoryFileForCRM($callAttachments, $index, $basic, $fileData, $call)
    {
        $filePath = $callAttachments[$index]->getFilename();
        $realPath = $basic . $filePath;
        $file = file_get_contents($realPath);
        $fileData['file'] = $file;
        $directory = $this->DIRECTORY_BASIC_UPLOAD . DIRECTORY_SEPARATOR . 'call_' . $call->getNrcall();
        $this->directoryCheck($directory);
        $uploadedFile = \Util\Utilities::factoryFromFileData($fileData, $directory, false);
        $fileData['filepath'] = $uploadedFile->getPathname();
        return $fileData;
    }

    public function getPageRequisite($nrsoliserv){

        $fileValues  = $this->INTRANETIntegration->getPageRequisite($nrsoliserv);
        if($fileValues){
            $this->setParamsForDownload($fileValues['extension'],null,$fileValues['name'],$fileValues['fileName'],false);
        }else{
            echo 'File not found';
        }
    }

    public static function directoryCheck($directory){
        if ( !is_dir($directory) && ! @mkdir($directory, 0777, true)) {
            throw new \InvalidArgumentException(sprintf(
                'O diretório "%s" não pode ser criado.',
                $directory
            ));
        }

        if ( ! is_writable($directory)) {
            throw new \InvalidArgumentException(sprintf(
                'O diretório "%s" não pode tem permissão de escrita.',
                $directory
            ));
        }
    }

    private function createDirectory($basic, array $directories){
        foreach($directories as $entity => $id){
            if(!file_exists($basic)){
                $this->directoryCheck($basic);
            }
            $basic .= DIRECTORY_SEPARATOR.$entity.'_'.$id;
        }
        return $basic;
    }

    public function uploadCallAttachments($callId, $description = null, array $documents, $createdBy,
                                          $issueId = null, $subject = null, $callType = null, $saveComment = true)
    {
        $connection = $this->entityManager->getConnection();
        
        if($description){
            if($saveComment){
                InstanceProvider::getService()->getCallComment()->saveProductCallComment($description, null,$callId,
                    $subject, $issueId, $callType);
            }
            $comentario = InstanceProvider::getRepository()->getComments()->findOneBy(array('task'=>$issueId),array('id'=>"DESC"));
        }else{
            $comentario = null;
        }
        try {
            $this->entityManager->beginTransaction();
            $filesUploaded = array();

            if($issueId){
                $cdOperador = Utilities::getEnvironment()->getCurrentUserId();
                if($comentario){
                    $pmAttachmentIds = $this->pmIntegrationService->uploadIssueAttachments($issueId, $documents, $cdOperador, $comentario->getId());
                }else{
                    $pmAttachmentIds = $this->pmIntegrationService->uploadIssueAttachments($issueId, $documents, $cdOperador);
                }
            }

            foreach($documents as $key => $fileData) {
                $filePath = $fileData['path'];
                $fileType = $fileData['type'];
                $fileLabel = $fileData['name'];
                $fileSize = $fileData['size'];

                $attachment = $this->callAttachmentFactory->createCallAttachment($callId, $fileLabel, $filePath, $fileType, $fileSize, $createdBy);

                if(!empty($pmAttachmentIds)){
                    $attachment->setCallRelationshipAttachment($pmAttachmentIds[$key]);
                }
                $this->entityManager->persist($attachment);
                $filesUploaded[] = $attachment;
            }
            $this->entityManager->flush();
            $connection->commit();

            return $filesUploaded;
        } catch (\Exception $e) {
            $connection->rollBack();
            return array();
        }
    }

    private function deleteCallAttachments($rows) {
        foreach($rows as $row) {
            /** @var \Teknisa\Model\Teknisa\Entities\CpCallAttachment $attachment */

            $attachment = $this->cpCallAttachmentRepository->find($row['ID']);
            try {
                unlink($attachment->getFilename());
            } catch (\Exception $e) {
                // If something goes wrong with unlink(), we should do nothing, only remove it from database
            }
            $this->entityManager->remove($attachment);
        }
        $this->entityManager->flush();
    }

    public function newCallAttachment($callId, $description, $documents, $createdBy, $callRelationship, $mainTypeCall, $subject = null, $callType = null)
    {

        if($callType == \Service\Call::TYPE_ISSUE || $callType == \Service\Call::TYPE_RICH_CLIENT){
            $this->addAttachmentsProductCall($callId, $description, $documents, $createdBy, $callRelationship, $subject, $callType);
        } else if ($callType == \Service\Call::TYPE_TICKET){
            $this->addAttachmentsSupportCall($description, $documents, $callRelationship);
        }
    }

    public function addAttachmentsSupportCall($description, $documents, $callRelationship)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $supportCall =  $this->cpCallRepository->findOneBy(array('callRelationship' => $callRelationship));
            if(!$supportCall){
                /** @var \Teknisa\Model\Crm\Entities\Contato $contato */
                $contato = $this->contatoRepository->findOneBy(array('nrseqcontato' => $callRelationship));
                $call = InstanceProvider::getFactory()->getCall()->createProductCall($contato->getDsrefecont(), $contato->getCdoperinclusao(),
                    $contato->getDsaberturacaso(), Call::TYPE_TICKET, $callRelationship, $contato->getCdproduto());

                $this->entityManager->persist($call);
                $this->entityManager->flush();
            }
            $nrorg = $this->environment->getCurrentOrganizationId();
            $createdBy = $this->environment->getCurrentUserId();
            $commentId = null;
            if($description){
                $comment = InstanceProvider::getService()->getCallComment()->saveSupportCallComment($description, $createdBy,
                    $callRelationship);
                $commentId = $comment['DADOS']['NRSEQEVOLCONT'];
            }
            $attachments = array();
            foreach ($documents as $key => $document) {
                $name = $document['name'];
                $arrayName = explode('.',$name);
                $type = end($arrayName);
                unset($arrayName[count($arrayName)-1]);
                $name = implode('.',$arrayName);
                $size = $document['size'];
                $documents[$key]['path'] = 'files/' . $documents[$key]['path'];
                $attachments[] = \Factory\Attachment::createArrayAttachmentTicket($createdBy,$documents,$type,
                    $name,$nrorg,$callRelationship,$size,$commentId);
            }
            Integration\CRMIntegration::addAttachmentTicketFromCRM($attachments);


            $this->entityManager->flush();

            $connection->commit();
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }

    }

    public function downLoadAttachment($attachmentId, $callAttachmentId)
    {
        /** @var \Teknisa\Model\Teknisa\Entities\CpCallAttachment $callAttachment */
        $callAttachment = $this->cpCallAttachmentRepository->findOneBy(array('nrcallattachment' => $callAttachmentId));
        /** @var Entities\Attachment $attachment */
        $attachment = $this->attachmentRepository->findOneBy(array('id' => $attachmentId));
        if ($attachment || $callAttachment) {
            list($name, $size, $type, $fileName) = $this->getParamsForDownload($attachment, $callAttachment);
            $this->setParamsForDownload($type, $size, $name, $fileName);
            exit;
        } else {
            throw new CustomException('ERR_NOT_ATTACHMENT');
        }
    }

    public function downLoadAttachmentTicket($attachmentId)
    {
        $CRMAnexocontato = $this->cpCallAttachmentRepository->downLoadAttachmentTicket($attachmentId);

        if ($CRMAnexocontato) {
            $name = $CRMAnexocontato['LABEL'];
            $size = $CRMAnexocontato['SIZEFILE'];
            $type = Utilities::getTypeByExtension($CRMAnexocontato['FILENAME']);
            $fileName = $CRMAnexocontato['FILENAME'];
            $fileServerFilesUrl = Utilities::getFileServerFilesUrl();
            $fileServerFilesUrl = str_replace('files/','',$fileServerFilesUrl);
            $fileName = $fileServerFilesUrl.$fileName;
            $this->setParamsForDownload($type, $size, $name, $fileName);
            exit;
        } else {
            throw new CustomException('ERR_NOT_ATTACHMENT');
        }
    }

    public function downLoadSpecification($issueId)
    {
        $attachmentId = $this->attachmentRepository->getAttachmentSpecification($issueId);
        if ($attachmentId) {
            /** @var Entities\Attachment $attachment */
            $attachment = $this->attachmentRepository->findOneBy(array('id' => $attachmentId));
            list($name, $size, $type, $fileName) = $this->getParamsForDownload($attachment);
            $this->setParamsForDownload($type, $size, $name, $fileName,false);
            exit;
        } else {
            throw new CustomException('ERR_NOT_ATTACHMENT');
        }
    }

    public function downloadBudgetSm($linkDocument)
    {
        if ($linkDocument) {
            $type = "application/pdf";
            $name = "Orçamento.pdf";
            $this->setParamsForDownload($type, null, $name, $linkDocument,false);
            exit;
        } else {
            throw new CustomException('ERR_NOT_ATTACHMENT');
        }
    }

    public function downLoadAttachmentGEP($fileName)
    {

        if ($fileName) {
            if(!file_exists($fileName)){
                $fileName = mb_convert_encoding($fileName, 'UTF-8', 'ISO-8859-1');
                if(!file_exists($fileName)){
                    throw new \Exception('Aquivo não existe');
                }
            }
            $arrayDirs = explode(DIRECTORY_SEPARATOR,$fileName);
            $this->setParamsForDownload('application/x-zip-compressed',null, array_pop($arrayDirs), $fileName);
            exit;
        } else {
            throw new \Exception('Nome do arquivo nulo');
        }
    }

    public function getSupportCallAttachmentsDataSet($callRelationship,$commentId = null)
    {
        $commentIdQuery = "";
        if($commentId) {
            $commentIdQuery = "AND EV.NRSEQEVOLCONT = " . $commentId;
        }
        $fileServerFilesUrl = Utilities::getFileServerFilesUrl();
        return $this->cpCallRepository->getAttachmentTicket($callRelationship,$fileServerFilesUrl,$commentIdQuery);
    }

    public function deleteSupportCallAttachments($rows)
    {
        foreach($rows as $row) {
            $attachment = $this->cpCallAttachmentRepository->find($row['ID']);

            $call = $this->cpCallRepository->find($attachment->getCall());
            $createdBy = $call->getReportedBy();
            $operador = $this->operadorRepository->findOneBy(array('cdoperador' => $createdBy));
            $contato = $this->contatoRepository->findOneBy(array('nrseqcontato' => $call->getCallRelationship()));
            $nrOrg = $operador->getNrorg();
            $cdFilial = $this->paramGeralRepository->findOneBy(array())->getCdfilial();
            $cdOperador = $createdBy;
            $nrContato = $contato->getNrcontato();
            $nrSeqEvolCont = $this->contatoRepository->getOpeningEvolContNrSeqEvolContFromContatoAndOpeningText($nrContato, \Service\Call::FIRST_COMMENTS);

            $this->crmIntegrationService->deleteAttachmentFromCRM($nrOrg, $cdFilial, $cdOperador, $nrContato, $nrSeqEvolCont, $attachment->getCallRelationshipAttachment());
            $this->deleteCallAttachments(array($row));
        }
    }

    public function responseQuestionIssue($callId, $description, $documents,
                                           $callRelationship = null, $subject = null,
                                          $callType = null){

        $createdBy = $this->environment->getCurrentUserId();
        $this->addAttachmentsProductCall($callId, $description, $documents, $createdBy,
            $callRelationship, $subject, $callType);

    }


    public function addAttachmentsProductCall($productCallId, $description, $documents, $createdBy, $callRelationship = null, $subject = null, $callType = null)
    {

        if($productCallId){
            $productCall = $this->cpCallRepository->find($productCallId);
            $callRelationship = $productCall->getCallRelationship();
        } else {
            $issue = $this->cpCallRepository->findIssueById($callRelationship);
            $productCall = InstanceProvider::getService()->getCall()->saveNewIssueTypeProductCall(\Service\Call::TYPE_ISSUE, $issue['REFERENCE'], $issue['PRODUCT'], $issue['CLASSIFICATION'], $issue['BOARD'], $issue['DESCRIPTION'], null, array(), $callRelationship);
            $productCallId = $productCall['ID'];
        }

        $this->uploadCallAttachments($productCallId, $description, $documents, $createdBy, $callRelationship, $subject, $callType);
    }

    public function addAttachmentsRichClient($productCallId, $description = null, $documents, $createdBy, $callRelationship = null)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $this->entityManager->beginTransaction();
            if(!$productCallId){
                $newProductCall = InstanceProvider::getService()->getCall()->saveNewRichClientTypeProductCallBySoliserv($callRelationship);
                $productCallId = $newProductCall['ID'];
            }
            $productCall = $this->cpCallRepository->find($productCallId);
            $callRelationship = $productCall->getCallRelationship();
            $subject = array();
            foreach ($documents as $fileData) {
                $subject[] = $fileData['name'];
            }

            if($description){
                $commentDate = $this->commentService->saveProductCallComment( $description, $this->environment->getCurrentUserId(), $productCallId,'');
            } else{
                $description = implode(', ',$subject);
                $commentDate = $this->commentService->saveProductCallComment( "Arquivos $description", $this->environment->getCurrentUserId(), $productCallId,'Adicionou anexo(s) pelo CP');
            }
            $dateString = $commentDate->format('YmdHis');
            $documents = $this->INTRANETIntegration->insertFile($documents, $productCall, $dateString);
            $this->uploadCallAttachments($productCallId, null, $documents, $createdBy);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
           throw $e;
        }

    }

    public function getCallAttachmentsDataSet($callId, $callRelationship, $callType, $mainTypeCall,
                                              $commentId = null,$filename = null)
    {
        $relationshipAttachments = array();
        if($callType == \Service\Call::TYPE_ISSUE || $callType == \Service\Call::TYPE_RICH_CLIENT){
            $relationshipAttachments = $this->getProductCallAttachmentsDataSet($callRelationship, $callType,
                $commentId,$filename);
        } else if ($callType == \Service\Call::TYPE_TICKET){
            $relationshipAttachments = $this->getSupportCallAttachmentsDataSet($callRelationship,$commentId);
        }

        return  $relationshipAttachments;
    }

    public function getProductCallAttachmentsDataSet($callRelationship, $callType, $commentId = null, $filename = null)
    {

        $relationshipAttachments = array();
        /** @var \Teknisa\Model\Teknisa\Entities\CpCall $productCall */
        $productCall = $this->cpCallRepository->findOneBy(array('callRelationship' => $callRelationship));
        $callId = empty($productCall) ? null : $productCall->getNrcall();
        $ticket = empty($productCall) ? null : $productCall->getTicketRelationship();

        if($callType == \Service\Call::TYPE_ISSUE) {
            $issueAttachments = $this->pmIntegrationService->getIssueAttachmentsDataSet($callRelationship,$commentId);
            foreach($issueAttachments as $index => $attachment) {
                $issueAttachments[$index]['__fromIssue'] = true;
            }
            $relationshipAttachments = $issueAttachments;
        } else{
            $relationshipAttachments = $this->INTRANETIntegration->getAttachments($callId,
                $callRelationship,$this->environment->getCurrentUser(),$filename);
        }
        if($ticket){
            $ticketAttachments = $this->getSupportCallAttachmentsDataSet($ticket);
            $relationshipAttachments = array_merge($relationshipAttachments,$ticketAttachments);
        }
        return  $relationshipAttachments;
    }

    public function deleteProductCallAttachments($idPMAttachment,$isIssue,$dir) {
        /** @var  \Teknisa\Model\Teknisa\Entities\CpCallAttachment $callAttachment */

        if ($isIssue) {
            $callAttachment = $this->cpCallAttachmentRepository->findOneBy(array('callRelationshipAttachment' => $idPMAttachment));
            $this->pmIntegrationService->deleteAttachment($idPMAttachment);
        } else {
            $callAttachment = $this->cpCallAttachmentRepository->findOneBy(array('nrcallattachment' => $idPMAttachment));
            if($callAttachment){
                $this->INTRANETIntegration->deleteAttachment($callAttachment);
            }else{
                $this->INTRANETIntegration->deleteFile($dir);
            }
        }
        if($callAttachment){
            $this->entityManager->remove($callAttachment);
            $this->entityManager->flush();
        }
    }

}

