<?php

namespace Service;

use Factory\FaturamentoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Util\Utilities;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Traits\TResponse;
use Service\BilletIssued;

class FaturamentoService
{
    use TResponse;

    protected $FaturamentoFactory;
    protected $EntityManager;
    protected $Environment;
    protected $newCode;
    protected $BilletIssued;

    public function __construct(FaturamentoFactory $FaturamentoFactory, Environment $Environment, NewCode $newCode, EntityManager $EntityManager, BilletIssued $BilletIssued)
    {
        $this->connection                   = $EntityManager->getConnection();
        $this->FaturamentoFactory           = $FaturamentoFactory;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
        $this->BilletIssued                 = $BilletIssued;
    }

    public function getFaturamento($parametro) {
        $retorno = (isset($_SESSION["_sf2_attributes"]["cdCiente"])) ? $this->FaturamentoFactory->getNrparcnegocio($_SESSION["_sf2_attributes"]["cdCiente"]) : 0;
        $operador = $this->Environment->getCdOperador();

        $dtinicompet = '';
        $dtfincompet = '';

        $dtinicompetFilter = '';
        $dtfincompetFilter = '';
        $dtinicompetFilterMed = '';
        $dtfincompetFilterMed = '';
        $qtMesesExibicao = -2;
        if($dtinicompet){
            $dtinicompetFilter = "AND (TRUNC(TO_DATE('{$dtinicompet}', 'DD/MM/YYYY')) < TRUNC(FAT.DTINICOMPET) OR TRUNC(TO_DATE('{$dtinicompet}', 'DD/MM/YYYY')) < TRUNC(FAT.DTFINCOMPET))";
            $dtinicompetFilterMed = "AND (TRUNC(TO_DATE('{$dtinicompet}', 'DD/MM/YYYY')) < TRUNC(MED.DTINICOMPET) OR TRUNC(TO_DATE('{$dtinicompet}', 'DD/MM/YYYY')) < TRUNC(MED.DTFINCOMPET))";

            /* Incluído este IF para definir a quantidade de meses que devem ser exibidos no Select (Através da variável $qtMesesExibicao).
                
               Anteriormente, estava fixo "-2", agora, se for filtrado a data inicial e a final, será a quantidade de meses entre essas datas.  Se não, continua pegando -2.*/
            if ($dtfincompet){

                $qtMesesExibicao = "((MONTHS_BETWEEN(TO_DATE('{$dtfincompet}', 'DD/MM/YYYY'), TO_DATE('{$dtinicompet}', 'DD/MM/YYYY'))+1)*-1)";
            }
        }
        if($dtfincompet){
            $dtfincompetFilter = "AND (TRUNC(TO_DATE('{$dtfincompet}', 'DD/MM/YYYY')) > TRUNC(FAT.DTINICOMPET) OR TRUNC(TO_DATE('{$dtfincompet}', 'DD/MM/YYYY')) > TRUNC(FAT.DTFINCOMPET))";
            $dtfincompetFilterMed = "AND (TRUNC(TO_DATE('{$dtfincompet}', 'DD/MM/YYYY')) > TRUNC(MED.DTINICOMPET) OR TRUNC(TO_DATE('{$dtfincompet}', 'DD/MM/YYYY')) > TRUNC(MED.DTFINCOMPET))";
        }
        
        $params = array(
            "NRORG" => $this->Environment->getNrOrg(),
            "NRPARCNEGOCIO" => $retorno[0]['NRPARCNEGOCIO']
        );

       return $this->FaturamentoFactory->getFaturamento($dtinicompetFilter, $dtfincompetFilter, $dtinicompetFilterMed,  $dtfincompetFilterMed, $qtMesesExibicao, $params);
    }

    public function getFaturamentoValorFixo($param) {
        $nrorg =  $this->Environment->getNrOrg();
        $params = Array(
            "NRPARCNEGOCIO" => $param['NRPARCNEGOCIO'],
            "VRORIGINAL"    => $param['VRORIGINAL'],
            "VRNEGCONTR"    => isset($param['VRNEGCONTR']) ? $param['VRNEGCONTR'] : null,
            "DTINICOMPET"   => $param['DTINICOMPET'],
            "DTFINCOMPET"   => $param['DTFINCOMPET'],
            "NRFATURACONT"  => $param['NRFATURACONT'], 
            "NRORG"         => $nrorg,
            "NRAGRUPAMENTO" => isset($param['NRAGRUPAMENTO']) ? $param['NRAGRUPAMENTO'] : null,
        );
        return $this->FaturamentoFactory->getFaturamentoValorFixo($params);
    }

    public function getFaturamentoNaoAgrupado($param) {
        $nrorg =  $this->Environment->getNrOrg();
        $params = Array(
            "NRPARCNEGOCIO" => $param['NRPARCNEGOCIO'],
            "VRORIGINAL"    => $param['VRORIGINAL'],
            "VRNEGCONTR"    => isset($param['VRNEGCONTR']) ? $param['VRNEGCONTR'] : null,
            "DTINICOMPET"   => $param['DTINICOMPET'],
            "DTFINCOMPET"   => $param['DTFINCOMPET'],
            "NRFATURACONT"  => $param['NRFATURACONT'], 
            "NRORG"         => $nrorg,
            "NRAGRUPAMENTO" => isset($param['NRAGRUPAMENTO']) ? $param['NRAGRUPAMENTO'] : null,
        );
        return $this->FaturamentoFactory->getFaturamentoNaoAgrupado($params);
    }

    public function getItensValorAgrupado($param) {
        $nrorg =  $this->Environment->getNrOrg();
        $params = Array(
            "NRPARCNEGOCIO" => $param['NRPARCNEGOCIO'],
            "VRORIGINAL"    => $param['VRORIGINAL'],
            "VRNEGCONTR"    => isset($param['VRNEGCONTR']) ? $param['VRNEGCONTR'] : null,
            "DTINICOMPET"   => $param['DTINICOMPET'],
            "DTFINCOMPET"   => $param['DTFINCOMPET'],
            "NRFATURACONT"  => $param['NRFATURACONT'], 
            "NRORG"         => $nrorg,
            "NRAGRUPAMENTO" => isset($param['NRAGRUPAMENTO']) ? $param['NRAGRUPAMENTO'] : null,
        );
        return $this->FaturamentoFactory->getItensValorAgrupado($params);
    }

    public function getItensValorAgrupadoEstimado($param) {
        $nrorg =  $this->Environment->getNrOrg();
        $params = Array(
            "NRPARCNEGOCIO" => $param['NRPARCNEGOCIO'],
            "VRORIGINAL"    => $param['VRORIGINAL'],
            "VRNEGCONTR"    => isset($param['VRNEGCONTR']) ? $param['VRNEGCONTR'] : null,
            "DTINICOMPET"   => $param['DTINICOMPET'],
            "DTFINCOMPET"   => $param['DTFINCOMPET'],
            "NRFATURACONT"  => $param['NRFATURACONT'], 
            "NRORG"         => $nrorg,
            "NRAGRUPAMENTO" => isset($param['NRAGRUPAMENTO']) ? $param['NRAGRUPAMENTO'] : null,
        );
        return $this->FaturamentoFactory->getItensValorAgrupadoEstimado($params);
    }

    public function findBilletIssuedFct($params) {
        $param = Array(
            "NRLANCTONF" => $params['NR']
        );
        
        $parametros =  $this->FaturamentoFactory->findBilletIssuedFct($param);
        
        $cdfilial = $parametros[0]['CDFILIAL'];
        $nrlanbolrec = $parametros[0]['NRLANBOLREC'];
        $cdreducont = $parametros[0]['CDREDUCONT'];
        $cdbanco = $parametros[0]['CDBANCO'];
        return $this->BilletIssued->printBillet($cdfilial, $nrlanbolrec, $cdreducont, $cdbanco);
    }

    public function getMedidasVariaveis($param)
    {   
        $dataSourceName = null;
        $orderBy        = null;
        $nrProdutoId    = $param[0]['value'];
        $nrOrgId        = $param[1]['value'];
        $cdServOrca     = $param[2]['value'];
        $page           = isset($param['page'])? intval($param['page']) : '';
        $pageSize       = isset($param['limit']) ? intval($param['limit'])  : '';

        $teste = $this->criarArrayformatado($param);
        $conditions = $this->filtraElementosArray($teste);
        $conditionsFixed = $this->fixConditionsToZmartFilterCriteria($conditions);
        $repositoryOmProduto = InstanceProvider::getRepository()->getOmProduto();
        $conditions = $conditionsFixed['CONDITIONS'];
        $zmartFilterCriteria = InstanceProvider::getUtilities()->convertFilterCriteriaToZmartFilterCriteria($conditions, $dataSourceName);
        
        if(empty($nrProdutoId)) {
            $productId = '0';
            $zmartFilterCriteria->addCondition('serviceId', '=', $cdServOrca);
        } else {
            $produto   = $repositoryOmProduto->find($nrProdutoId);
            $productId = strval($produto->getNrexternalProductId());
        }
        
        $zmartFilterCriteria->addCondition('productId', '=', $productId); // VALOR FIXADO ESTATICAMENTE, ALTERAR QUANDO FINALIZAR TAREFA ( $productId )
        $zmartFilterCriteria->setPage($page);
        $zmartFilterCriteria->setPageSize($pageSize);

        if (empty($orderBy)) {
            $zmartFilterCriteria->addOrderBy('date', FilterCriteria::ORDER_DESC);
        } else {
            foreach ($orderBy as $idx => $order) {
                $zmartFilterCriteria->addOrderBy($idx, $order);
            }
        }
       
        $criteria = InstanceProvider::getMongoQueryBuilder()->buildQueryFromFilterCriteria($zmartFilterCriteria);
        $this->setMappedLikeAllCriteria($criteria, $conditionsFixed['MAPPED_LIKE_ALL']);
        $registers = InstanceProvider::getLicenseMongoDB()->aggregate("VariableMeasure_{$nrOrgId}", $criteria);
        
        foreach ($registers as &$register) {
            $register['date']           = \DateTime::createFromFormat('Ymd', $register['date'])          ->format('d/m/Y');
            $register['endCompetence']  = \DateTime::createFromFormat('Ymd', $register['endCompetence']) ->format('d/m/Y');
            $register['initCompetence'] = \DateTime::createFromFormat('Ymd', $register['initCompetence'])->format('d/m/Y');
            $contractId  = Utilities::getValueInFilterDataSourceByName($conditions, 'contractId');
            $register['countMin'] = $repositoryOmProduto->getMinContract($contractId, $nrProdutoId, $cdServOrca);
        }
     
        $retorno = (object) array(
            'data' => $registers,
            'pagination' => (object) array( 'page' => $page, 'limit' =>$pageSize)
        );
        return $retorno;
    }

    public function criarArrayformatado($conditions) {
        $retorno = [];
        foreach ($conditions as  $value) {
          if(is_array($value)){
            array_push($retorno, $value);
          }
        }
        return $retorno;
    }

    public function filtraElementosArray($param){
        $conditions = $param;
        foreach ($conditions as $key => $value) {
            if ($value['columnName'] == 'NRPRODUTO_ID' || $value['columnName'] == 'NRORG_ID' || $value['columnName'] == 'CDSERVORCA' ) {
                unset($conditions[$key]);
            }
        }
        return $conditions;
    }

    private function fixConditionsToZmartFilterCriteria($conditions) {
        $mappedLikeAll = null;
            foreach($conditions as $idx => &$condition) {
                    $name = isset($condition["name"]) ? $condition["name"] : $condition["columnName"];
                    if($condition['operator'] == 'LIKE_I' || $condition['operator'] == 'LIKE') {
                        $condition['operator'] = 'LIKE_ALL';
                    }
                    if($name == 'date') {
                        $condition['value'] = array(
                            intval(\DateTime::createFromFormat('d/m/Y', $condition['value'][0])->format('Ymd')),
                            intval(\DateTime::createFromFormat('d/m/Y', $condition['value'][1])->format('Ymd'))
                        );
                    } else if($name == 'initCompetence' || $name == 'endCompetence'){
                        $condition['value'] = intval(\DateTime::createFromFormat('d/m/Y H:i:s', $condition['value'])->format('Ymd'));
                    }
                    if ($condition['operator'] == 'MAPPED_LIKE_ALL') {
                        $mappedLikeAll = $condition;
                        unset($conditions[$idx]);
                    }
            }
        return array('CONDITIONS' => $conditions, 'MAPPED_LIKE_ALL' => $mappedLikeAll);
    }

    private function setMappedLikeAllCriteria(&$criteria, $mappedLikeAll) {
        if(!empty($mappedLikeAll)) {
            if (!isset($criteria[0]['$match'])) {
                $criteria['$match'] = array();
            }
            $name = isset($mappedLikeAll['name']) ? $mappedLikeAll['name'] : $mappedLikeAll['columnName'];
            $mappedLikeAllColumns = array('measureName', 'contractId', 'count', 'operatorName', 'serviceName');
            $value = str_replace('%', '', $mappedLikeAll['value'][0]['_ALL']);
            foreach ($mappedLikeAllColumns as $column) {
                $criteria[0]['$match']['$or'][] = array($column => array('$regex' => ".*{$value}.*"));
            }
        }
    }
}
