<?php

namespace Service;

use Doctrine\ORM\EntityManager;
use Exception\CustomException;
use Teknisa\Model\Geral\Entities;
use Util\Utilities;
use Util\InstanceProvider;

class Notification
{
    /** @var \Doctrine\ORM\EntityManager $entityManager */
    protected $entityManager;
    /** @var \Util\Utilities*/
    private $environment;


    public function __construct(EntityManager $entityManager,\Util\Utilities $environment)
    {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
    }

    public function getNotifications(){
        $operador   = $this->environment->getCurrentUser();
        $cdOperador = $operador->getCdoperador();
        $query =    "SELECT IC.NRINFECOMMERCE,
                    IC.DSTITULOINF, IC.TXCONTEUDOINF, IC.IDNOTIFICACAO
                    FROM INFORMATIVOECOMMERCE IC
                    WHERE NOT EXISTS (
                        SELECT IO.NRINFECOMMERCE FROM INFORMATIVOECOMOPER IO
                        WHERE IO.CDOPERADOR = :CDOPERADOR
                        AND IO.NRINFECOMMERCE = IC.NRINFECOMMERCE
                    )
                    AND TRUNC(SYSDATE) BETWEEN TRUNC(IC.DTINFORMARIVI) AND TRUNC(IC.DTEXPIRAINFO)";
        $params = array(
            "CDOPERADOR" => $cdOperador
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }

    public function readNotification($notificationId){
        $connection = $this->entityManager->getConnection();
        try{
            $connection->beginTransaction();
            $operador   = $this->environment->getCurrentUser();
            $cdOperador = $operador->getCdoperador();
            $nrOrg = $operador->getNrorg();
            $newCodeService = InstanceProvider::getService()->getNewCode();
            $nrinfecomoper = $newCodeService->getCodeByTableName('INFORMATIVOECOMOPER', $nrOrg);
            $query =    "INSERT INTO INFORMATIVOECOMOPER (NRINFECOMOPER, NRINFECOMMERCE, CDOPERADOR)
                        VALUES (:NRINFECOMOPER, :NRINFECOMMERCE, :CDOPERADOR)";
            $params = array(
                "NRINFECOMOPER" => $nrinfecomoper,
                "NRINFECOMMERCE" => $notificationId,
                "CDOPERADOR" => $cdOperador
            );
            $connection->executeQuery($query, $params);
            $this->entityManager->flush();
            $connection->commit();
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

}
