/* Sistemas Integrados TECNISA
   Planejamento Alimentao Industrial

Nome: NUTRIENTE_DO_CARDAPIO_PLA

Descrio:
Clculo dos Nutrientes do Cardapio e gravao na tabela NUTRTRAB.

P_FILIAL     - Codigo da Filial.
P_SERVICOS   - String contendo sequencia de Servios a serem calculados.
P_NUTRIENTES - String contendo Nutrientes a serem calculados.
P_NRSEQUNUTR - Nmero sequencial a ser utilizado para gravao dos dados.
P_IDNDPCAL   - Indicador de calculo do NDPCal.
P_DTINICIAL  - Data inicial.
P_DTFINAL    - Data final.

Criao:
02/03/98 - Zed

Manutenes:
05/03/99 - Ederson - Reviso geral.
****************************************************************
             CORREO DO CLCULO DO NDPCAL%
****************************************************************
A frmula usada para clculo do NDPCal% :

NDPCAL% = NPU * Sum(Protenas) * 4
          ------------------------
                    VCT

onde:

NPU = Somatrio das protenas dos componentes da receita multiplicados
      pelo Fator de Clculo de NDPCAL, informado na tela de parametrizao
      por grupo de produtos, sendo 0,7 para alimentos de origem animal;
      0,6 para leguminosas e 0,5 para cereais, frutas e outros vegetais.
Sum(Protenas) = Somatrio das protenas dos componentes da receita
VCT = "Valor Calrico Total" = Soma(carboidratos) x 4 +
                               Soma(lipdeos)    x 9  +
                               Soma(protenas)   x 4  +
                               Soma(lcoois)     x 7
****************************************************************

*/

IF EXISTS(SELECT name FROM sysobjects
          WHERE name = 'NUTRIENTE_DO_CARDAPIO_PLA' AND type = 'P')
  DROP PROCEDURE NUTRIENTE_DO_CARDAPIO_PLA
  GO

CREATE PROCEDURE NUTRIENTE_DO_CARDAPIO_PLA @P_FILIAL       VARCHAR(4),
                                           @P_SERVICOS     VARCHAR(2000),
                                           @P_NUTRIENTES   VARCHAR(2000),
                                           @P_NRSEQUNUTR   VARCHAR(10),
                                           @P_IDNDPCAL     VARCHAR,
                                           @P_DTINICIAL    DATETIME,
                                           @P_DTFINAL      DATETIME

AS
       DECLARE @V_NRSEQUNUTR   VARCHAR(10);
       DECLARE @V_CDFILINUTR   VARCHAR(4);
       DECLARE @V_CDSERVNUTR   VARCHAR(3);
       DECLARE @V_DTREFENUTR   DATETIME;
       DECLARE @V_CDDIETNUTR   VARCHAR(4);
       DECLARE @V_NRSEQPRATNUT SMALLINT;
       DECLARE @V_CDTPPRATNUTR VARCHAR(2);
       DECLARE @V_CDFAPRATNUTR VARCHAR(4);
       DECLARE @V_CDPRATNUTR   VARCHAR(10);
       DECLARE @V_CDPRODNUTR   VARCHAR(10);
       DECLARE @V_QTCARBOIDRATO NUMERIC(12,5);
       DECLARE @V_QTPROTEINA    NUMERIC(12,5);
       DECLARE @V_QTLIPIDEO     NUMERIC(12,5);
       DECLARE @V_QTALCOOL      NUMERIC(12,5);
       DECLARE @V_QTCALORIA     NUMERIC(12,5);
BEGIN

   INSERT INTO NUTRTRAB
      (NRSEQUNUTR,
       CDFILINUTR,
       CDSERVNUTR,
       DTREFENUTR,
       CDDIETNUTR,
       NRSEQPRATNUT,
       CDTPPRATNUTR,
       CDFAPRATNUTR,
       CDPRATNUTR,
       CDPRODNUTR,
       CDNUTRCALC,
       QTPRODNUTR,
       QTNUTRCALC,
       QTPROTLIQU,
       VRCALORNUTR,
       VRPECONSNUTR,
       SGUNIDPRODT,
       QTPRODPRATT)
   SELECT
      @P_NRSEQUNUTR,
      CD.CDFILIAL,
      CD.CDSERVFILI,
      CD.DTCARDAPIO,
      ISNULL(CD.CDDIETPAD, 'XXXX'),
      CC.NRSEQPRATCAR,
      CC.CDTIPOPRAT,
      CC.CDFILAUXPRAT,
      CC.CDPRATO,
      RS.CDPRODUTO,
      NP.CDNUTRIENTE,
      ISNULL(RS.QTCALCNUTRSE,0) AS QTPRODUTO,
      ISNULL(NP.QTNUTRPROD * 10 * ISNULL((CASE PRODFILI.VRPESOUNIDFI WHEN 0 THEN PR.VRPESOUNID ELSE PRODFILI.VRPESOUNIDFI END), PR.VRPESOUNID) * RS.QTCALCNUTRSE, 0),
      RS.QTPRODRECSER AS QTPRODLIQ,
      0,
      ISNULL(CC.VRPECONSCARD,0) VRPECONSCARD,
      RS.SGUNIDADE AS SGUNIDPROD,
      (RS.VRPEFATNUTSE * RS.QTBRUTASEUNR) /100 AS QTPRODPRATT
   FROM
      CARDADIA CD, COMPCARD CC, RECESERV RS, NUTRPROD NP, PRODUTO PR, PRODFILI
   WHERE
      (CD.CDFILIAL = @P_FILIAL)                           AND
      (CHARINDEX(CD.CDSERVFILI, @P_SERVICOS) <> 0)        AND
      (CD.DTCARDAPIO BETWEEN @P_DTINICIAL AND @P_DTFINAL) AND
      (CD.CDFILIAL        = CC.CDFILIAL)                  AND
      (CD.NRCARDAPIO      = CC.NRCARDAPIO)                AND
      (CD.CDFILIAL        = RS.CDFILIAL)                  AND
      (CD.CDSERVFILI      = RS.CDSERVFILI)                AND
      (CC.CDFILAUXPRAT    = RS.CDFILAUXPRAT)              AND
      (CC.CDPRATO         = RS.CDPRATO)                   AND
      (PRODFILI.CDFILIAL  = @P_FILIAL)                    AND
      (PRODFILI.CDPRODUTO = PR.CDPRODUTO)                 AND
      (CHARINDEX(NP.CDNUTRIENTE, @P_NUTRIENTES) <> 0)     AND
      (RS.CDPRODUTO = NP.CDPRODUTO)                       AND
      (RS.CDPRODUTO = PR.CDPRODUTO)
   UNION
   SELECT
      @P_NRSEQUNUTR,
      CD.CDFILIAL,
      CD.CDSERVFILI,
      CD.DTCARDAPIO,
      ISNULL(CD.CDDIETPAD, 'XXXX'),
      CC.NRSEQPRATCAR,
      CC.CDTIPOPRAT,
      CC.CDFILAUXPRAT,
      CC.CDPRATO,
      RF.CDPRODUTO,
      NP.CDNUTRIENTE,
      ISNULL(RF.QTCALCNUTRLO,0) AS QTPRODUTO,
      ISNULL(NP.QTNUTRPROD * 10 * ISNULL((CASE PRODFILI.VRPESOUNIDFI WHEN 0 THEN PR.VRPESOUNID ELSE PRODFILI.VRPESOUNIDFI END), PR.VRPESOUNID) * RF.QTCALCNUTRLO, 0),
      RF.QTPRODRECLOC AS QTPRODLIQ,
      0,
      ISNULL(CC.VRPECONSCARD,0) VRPECONSCARD,
      RF.SGUNIDADE AS SGUNIDPROD,
      (RF.VRPEFATNUTLO * RF.QTBRUTALOUNR) /100 AS QTPRODPRATT
   FROM
      CARDADIA CD, COMPCARD CC, RECEFILI RF, NUTRPROD NP, PRODUTO PR, PRODFILI
   WHERE
      (CD.CDFILIAL = @P_FILIAL)                           AND
      (CHARINDEX(CD.CDSERVFILI, @P_SERVICOS) <> 0)        AND
      (CD.DTCARDAPIO BETWEEN @P_DTINICIAL AND @P_DTFINAL) AND
      (CD.CDFILIAL        = CC.CDFILIAL)                  AND
      (CD.NRCARDAPIO      = CC.NRCARDAPIO)                AND
      (CD.CDFILIAL        = RF.CDFILIAL)                  AND
      (CC.CDFILAUXPRAT    = RF.CDFILAUXPRAT)              AND
      (CC.CDPRATO         = RF.CDPRATO)                   AND
      (PRODFILI.CDFILIAL  = @P_FILIAL)                    AND
      (PRODFILI.CDPRODUTO = PR.CDPRODUTO)                 AND
      (CHARINDEX(NP.CDNUTRIENTE, @P_NUTRIENTES) <> 0)     AND
      (RF.CDPRODUTO = NP.CDPRODUTO)                       AND
      (RF.CDPRODUTO = PR.CDPRODUTO)                       AND
      NOT EXISTS (SELECT
                     CDPRATO
                  FROM
                     RECESERV
                  WHERE
                     (CDFILAUXPRAT = CC.CDFILAUXPRAT) AND
                     (CDPRATO      = CC.CDPRATO)      AND
                     (CDFILIAL     = CD.CDFILIAL)     AND
                     (CDSERVFILI   = CD.CDSERVFILI)
                  )
   UNION
   SELECT
      @P_NRSEQUNUTR,
      CD.CDFILIAL,
      CD.CDSERVFILI,
      CD.DTCARDAPIO,
      ISNULL(CD.CDDIETPAD, 'XXXX'),
      CC.NRSEQPRATCAR,
      CC.CDTIPOPRAT,
      CC.CDFILAUXPRAT,
      CC.CDPRATO,
      RP.CDPRODUTO,
      NP.CDNUTRIENTE,
      ISNULL(RP.QTCALCNUTRPR,0) AS QTPRODUTO,
      ISNULL(NP.QTNUTRPROD * 10 * ISNULL((CASE PRODFILI.VRPESOUNIDFI WHEN 0 THEN PR.VRPESOUNID ELSE PRODFILI.VRPESOUNIDFI END), PR.VRPESOUNID) * RP.QTCALCNUTRPR, 0),
      RP.QTPRODPRAT AS QTPRODLIQ,
      0,
      ISNULL(CC.VRPECONSCARD,0) VRPECONSCARD,
      RP.SGUNIDADE AS SGUNIDPROD,
      (RP.VRPEFATNUTPR * RP.QTBRUTAPRUNR) /100 AS QTPRODPRATT
   FROM
      PRODFILI, NUTRPROD NP, PRODUTO PR, CARDADIA CD, COMPCARD CC, RECEPADR RP
   WHERE
      (CD.CDFILIAL = @P_FILIAL)                           AND
      (CHARINDEX(CD.CDSERVFILI, @P_SERVICOS) <> 0)        AND
      (CD.DTCARDAPIO BETWEEN @P_DTINICIAL AND @P_DTFINAL) AND
      (CD.CDFILIAL        = CC.CDFILIAL)                  AND
      (CD.NRCARDAPIO      = CC.NRCARDAPIO)                AND
      (RP.CDFILAUXPRAT    = 'P')                          AND
      (CC.CDPRATO         = RP.CDPRATO)                   AND
      (PRODFILI.CDFILIAL  = @P_FILIAL)                    AND
      (PRODFILI.CDPRODUTO = PR.CDPRODUTO)                 AND
      (CHARINDEX(NP.CDNUTRIENTE, @P_NUTRIENTES) <> 0)     AND
      (RP.CDPRODUTO = NP.CDPRODUTO)                       AND
      (RP.CDPRODUTO = PR.CDPRODUTO)                       AND
      NOT EXISTS (SELECT
                     CDPRATO
                  FROM
                     RECEFILI
                  WHERE
                     (CDFILAUXPRAT = CC.CDFILAUXPRAT) AND
                     (CDPRATO      = CC.CDPRATO)      AND
                     (CDFILIAL     = CD.CDFILIAL)
                  ) AND
      NOT EXISTS (SELECT
                     CDPRATO
                  FROM
                     RECESERV
                  WHERE
                     (CDFILAUXPRAT = CC.CDFILAUXPRAT) AND
                     (CDPRATO      = CC.CDPRATO)      AND
                     (CDFILIAL     = CD.CDFILIAL)     AND
                     (CDSERVFILI   = CD.CDSERVFILI)
                  );

   /* ------------------------- CALCULO DO DNPCal --------------------------- */
   IF @P_IDNDPCAL = 'S'
   BEGIN
      /* ---------------- GRAVA O VALOR DE PROTEINA LIQUIDA ----------------- */
      /*UPDATE
         NUTRTRAB
      SET
         QTPROTLIQU = (SELECT
                          ISNULL(MAX(VRFATCALNDPC) * QTNUTRCALC,0)
                       FROM
                          NDPCAL
                       WHERE
                          CDPRODUTO = (SELECT
                                          MAX(CDPRODUTO)
                                       FROM
                                          NDPCAL
                                       WHERE
                                          CHARINDEX(NUTRTRAB.CDPRODNUTR,CDPRODUTO) = 1
                                      )
                       )
      WHERE
         CDNUTRCALC = (SELECT MAX(CDPROTEINA) FROM PARAMGERAL) AND
         NRSEQUNUTR = @P_NRSEQUNUTR;
      */
      /* --------------------- GRAVA O VALOR DE CALORIAS -------------------- */
      UPDATE
         NUTRTRAB
      SET
         VRCALORNUTR = QTNUTRCALC
      WHERE
         CDNUTRCALC = (SELECT MAX(CDCALORIAS) FROM PARAMGERAL) AND
         NRSEQUNUTR = @P_NRSEQUNUTR;

   END;
END;
GO


-- VERSAO --
EXEC ATUVEROBJ 'NUTRIENTE_DO_CARDAPIO_PLA','5.05.014';
GO

