import { Utilities } from '@/util/Utilities';
import { LoadingService, Modal, ModalService, Text } from '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';
import { TekGrid, TekTreeGrid, TekGridColumn } from '@zeedhi/teknisa-components-common';

export class FaturamentoController {
    public visibilidadeFrameGridPincipal: Boolean = true;

    public visibilidadeItensFrameItensValorFixo: Boolean = false;

    public visibilidadeItensFrameItensValorVariavel: Boolean = false;

    public modalMedidaVariavel!: Modal;

    public route: String = '';

    public currentRow: any;

    public classModal: String = '';

    public gridHeight: String = '';

    private verificar: Boolean = true;

    public visibilidadeAction: Boolean = false;

    public async exibirModalFilter({ row }: any) {
        this.modalMedidaVariavel = await ModalService.createFromJson('faturamento/modal/modalMedidaVariavel', true);
        this.modalMedidaVariavel.show();
        this.currentRow = row;
    }

    public onMountedmodal() {
        const row = this.currentRow;
        this.popularModalMedidasVariaveis(row);
        this.popularModalGridMedidaVariavel(row);
    }

    public onBeforeMountModal() {
        this.classModal = 'modalVariableMeasure';
    }

    public fecharModalFilter() {
        this.modalMedidaVariavel.hide();
        this.modalMedidaVariavel?.destroy();
    }

    public controleVisibilidadeFrame(component: any, row: any) {
        if (row !== undefined && row.IDTIPO === 'P') {
            this.visibilidadeFrameGridPincipal = false;
            this.visibilidadeItensFrameItensValorFixo = true;
            this.visibilidadeItensFrameItensValorVariavel = false;
            this.setFiltroValorFixo(row);
        } else if (row !== undefined && row.IDTIPO === 'A') {
            this.visibilidadeFrameGridPincipal = false;
            this.visibilidadeItensFrameItensValorFixo = false;
            this.visibilidadeItensFrameItensValorVariavel = true;
            this.setFiltroValorAgrupado(row);
        } else if (row !== undefined && row.IDTIPO === 'AME') {
            this.visibilidadeFrameGridPincipal = false;
            this.visibilidadeItensFrameItensValorFixo = false;
            this.visibilidadeItensFrameItensValorVariavel = true;
            this.setFiltroValorAgrupadoEstimado(row);
        } else if (row !== undefined && row.IDTIPO === 'M') {
            this.visibilidadeFrameGridPincipal = false;
            this.visibilidadeItensFrameItensValorFixo = false;
            this.visibilidadeItensFrameItensValorVariavel = true;
            this.setFiltroValorNaoAgrupado(row);
        }
        if (component.name === 'botaoVoltarGridFixo' || component.name === 'botaoVoltarGridVariavel') {
            this.visibilidadeFrameGridPincipal = true;
            this.visibilidadeItensFrameItensValorFixo = false;
            this.visibilidadeItensFrameItensValorVariavel = false;
            this.limparDatasourceGrids();
        }
    }

    public limparDatasourceGrids() {
        Metadata.getInstance<TekGrid>('gridItensValorFixo').datasource.data = [];
        Metadata.getInstance<TekGrid>('gridMedidasVariaveis').datasource.data = [];
    }

    public setFiltroValorFixo(row: any) {
        const grid = Metadata.getInstance<TekGrid>('gridItensValorFixo');
        grid.datasource.setFilter({
            NRPARCNEGOCIO: row.NRPARCFATURA,
            VRORIGINAL: row.VRORIGINAL,
            VRNEGCONTR: row.VRNEGCONTR,
            DTINICOMPET: row.DTINICOMPET,
            DTFINCOMPET: row.DTFINCOMPET,
            NRFATURACONT: row.NRFATURACONT,
        });
    }

    public setFiltroValorAgrupado(row: any) {
        const grid = Metadata.getInstance<TekGrid>('gridItensValorVariavel');
        this.route = '/getItensValorAgrupado';
        grid.datasource.setFilter({
            NRPARCNEGOCIO: row.NRPARCFATURA,
            VRORIGINAL: row.VRORIGINAL,
            VRNEGCONTR: row.VRNEGCONTR,
            DTINICOMPET: row.DTINICOMPET,
            DTFINCOMPET: row.DTFINCOMPET,
            NRFATURACONT: row.NRFATURACONT,
        });
    }

    public setFiltroValorNaoAgrupado(row: any) {
        const grid = Metadata.getInstance<TekTreeGrid>('gridItensValorVariavel');
        this.route = '/getItensValorNaoAgrupado';
        grid.datasource.setFilter({
            NRPARCNEGOCIO: row.NRPARCFATURA,
            VRORIGINAL: row.VRORIGINAL,
            VRNEGCONTR: row.VRNEGCONTR,
            DTINICOMPET: row.DTINICOMPET,
            DTFINCOMPET: row.DTFINCOMPET,
            NRFATURACONT: row.NRFATURACONT,
        });
    }

    public setFiltroValorAgrupadoEstimado(row: any) {
        const grid = Metadata.getInstance<TekTreeGrid>('gridItensValorVariavel');
        this.route = '/getItensValorAgrupadoEstimado';
        grid.datasource.setFilter({
            NRPARCNEGOCIO: row.NRPARCFATURA,
            VRORIGINAL: row.VRORIGINAL,
            VRNEGCONTR: row.VRNEGCONTR,
            DTINICOMPET: row.DTINICOMPET,
            DTFINCOMPET: row.DTFINCOMPET,
            NRFATURACONT: row.NRFATURACONT,
        });
    }

    public getMedidasVariaveis({ row }: any) {
       if (row.tree__level !== 1 || (row.tree__level === 1 && row.tree__children.length === 0)) {
        const grid = Metadata.getInstance<TekGrid>('gridMedidasVariaveis');
        const conditions = [
            { columnName: 'NRPRODUTO_ID', operator: '=', value: (row.IDELEMENTO !== null) ? row.IDELEMENTO.replace('P_', '') : null },
            { columnName: 'NRORG_ID', operator: '=', value: 1158 },
            { columnName: 'CDSERVORCA', operator: '=', value: row.CDSERVORCA },
            { columnName: 'initCompetence', operator: '=', value: row.DTINICOMPET },
            { columnName: 'endCompetence', operator: '=', value: row.DTFINCOMPET },
            { columnName: 'contractId', operator: '=', value: row.NRSEQCONTRATO },
        ];
        grid.datasource.setFilter(conditions);
       }
    }

    public rowStyleConditions(row: any) {
        switch (row.IDSITUACAO) {
            case 'ATRASO':
                return { cssClass: 'linhaGridVermelha' };
            case 'PLANEJADO':
                return { cssClass: 'linhaGridAzul' };
            case 'PENDENTE':
                return { cssClass: 'linhaGridAmarelo' };
            default:
                return { cssClass: 'linhaGridVerde' };
        }
    }

    public visibilidadeVrNegociado({ row }: any) {
        if (row.VRNEGCONTR !== undefined && row.VRNEGCONTR !== null && row.VRNEGCONTR !== '0') {
            this.visibilidadeAction = true;
            return true;
        }
        this.visibilidadeAction = false;
        return false;
    }

    // public setFilterGridFaturamento() {
    //     const grid = Metadata.getInstance<TekGrid>('gridFaturamento');
    //     grid.datasource.setFilter({
    //         CDCLIENTE: localStorage.getItem('cdClienteLogin'),
    //     });
    // }

    public visibilidadeBotao({ row }: any) {
        if (row.IDSITUACAO === 'FATURADA' || row.IDSITUACAO === 'ATRASO') {
            return true;
        }
        return false;
    }

    public imprimirBoleto(row: any) {
        LoadingService.show();
        Http.post('/findBilletIssuedFct', {
            NR: row.NRLANCTONF,
        }).then((response) => {
            if (response.length) {
                response[0].changed = true;
                window.open(response);
                console.warn(response);
            } else {
                Utilities.showDialog('Atenção!', 'warning', 'O boleto ainda não foi gerado.');
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public popularModalMedidasVariaveis(row: any) {
        Metadata.getInstance<Text>('medidaVariavelText').text = row.measureName;
        Metadata.getInstance<Text>('qtdMinimaText').text = row.countMin;
        Metadata.getInstance<Text>('qtdFaturamentoText').text = row.count;
        Metadata.getInstance<Text>('dataColetaText').text = row.date;
        Metadata.getInstance<Text>('inicioVigenciaText').text = row.initCompetence;
        Metadata.getInstance<Text>('finalVigenciaText').text = row.endCompetence;
    }

    public popularModalGridMedidaVariavel(row: any) {
        const grid = Metadata.getInstance<TekGrid>('gridDadosColetados');
        this.modularGridmodal(row.log, grid);
        grid.datasource.updateData(row.log);
    }

    public modularGridmodal(row: any, grid: any) {
        if (row[0] !== undefined) {
            const rows = row[0];
            grid.columns = Object.keys(rows).map((key) => new TekGridColumn({ name: key, label: key }, grid));
        }
    }

    public estenderModal() {
        if (this.classModal === 'modalVariableMeasure') {
            this.classModal = 'modalVariableMeasureExtend';
            this.gridHeight = '440px';
        } else {
            this.classModal = 'modalVariableMeasure';
            this.gridHeight = '190px';
        }
    }

    public clickGrid({ component, row }: any) {
        if (this.verificar) {
            this.verificar = false;
            if (component.name === 'gridFaturamento' || component.name === 'botaoVoltarGridFixo' || component.name === 'botaoVoltarGridVariavel') {
                this.controleVisibilidadeFrame(component, row);
            } else {
                this.imprimirBoleto(row);
            }
            setTimeout(() => {
                this.verificar = true;
            }, 500);
        }
    }
}
